<?php
/**
 * Youtube api class file v3
 *
 * @package    VideoGlue
 * @author     Original Author <xlordt@gmail.com>
 * @copyright  2014-2015 hexcoderz.net
 * @link       http://www.hexcoderz.net
 */

/**
 * Class Namespace
 */
//namespace classes\yt;

/**
 * Name of our class youtube_v3_api
 */
class pvg_youtubeapi {

    /**
     * Holds the information returned by youtube
     *
     * @var array $results
     */
    private $results;

    /**
     * Holds the type of youtube query. Parse type with url
     *
     * @var string $urlParts
     */
    private $youtubeType;

    /**
     * Holds the youtube channel ID/Username
     *
     * @var string $youtubeId
     */
    private $youtubeId;

    /**
     * Holds multiple youtube ID
     *
     * @var array $youtubeMultiId
     */
    private $youtubeMultiId = array();

    /**
     * Youtube pagination, Used to query max results per query
     *
     * @var int $maxResults
     */
    private $maxResults = 50;

    /**
     * Youtube pagination, Used to query min result per query
     *
     * @var int $minResult ;
     */
    private $minResult = 0;

    /**
     * Get previous page token.
     *
     * @var $prevPageToken
     */
    private $prevPageToken;

    /**
     * Defines youtube development key (Use this one or change to your own)
     *
     * @var string $devKey
     */
    private $devKey = "AIzaSyApT7uuKAujVIAham_nvruAnp8o7a1tlCA";

    /**
     * Defines youtbue api V3 link that will be used to connect to
     * the service
     *
     * @var string $apiLink
     */
    private $apiLink = "https://www.googleapis.com/youtube/v3/";

    /**
     * Make sure we have a token being sent (will implement later)
     *
     * @var string $ytToken
     */
    private $ytToken;

    /**
     * Used to get the http information when sending an api request.
     * This will tell us if the request has returned with a proper http_code
     *
     * @var array $serviceInfo
     */
    private $serviceInfo;

    /**
     * Defines the type of query we want to execute, currently the only
     * supported are.....
     *
     * + videos
     * + channnels
     * + playlists
     *
     * @var array $searchType
     */
    private $searchType = array(
        "search"   => array("search", "q", "part=id,snippet"),
        "video"    => array("videos", "id", "part=snippet,statistics,contentDetails,status"),
        "channel"  => array("channels", "id", "part=snippet,status,contentDetails"),
        "playlist" => array("playlistItems", "playlistId", "part=snippet,status,contentDetails"),
        "favorite" => array("playlistItems", "playlistId", "part=snippet,status,contentDetails"),
    );

    private $orderBy = array(
        "date", "rating", "relevance", "title", "videoCount", "viewCount"
    );

    /**
     * Our construct, the heart of this class
     *
     * @param NULL|int     $id
     * @param NULL|string  $type
     * @param NULL|string  $prevPageToken
     * @param NULL|string  $nextPageToken
     * @param NULL|string  $orderBy
     */
    public function __construct($id = NULL, $type = NULL, $prevPageToken = NULL, $nextPageToken = NULL, $orderBy = NULL) {

        if( is_array($id) && !is_null($type) && array_key_exists($type, $this->getSearchOptions()) ) {

            $this->setMultipleYTIds($id)->sendMultiRequest();

        }else if( !empty($id) && !is_null($type) && array_key_exists($type, $this->getSearchOptions())) {

            $this->setParseYouTubeParams($type, urlencode($id), $prevPageToken, $nextPageToken)->sendRequest();
        }

        return FALSE;
    }

    /**
     * Parse youtube parameters
     *
     * @param null|string $type
     * @param null|string $id
     * @param null|string $prevPageToken
     * @param null|string $nextPageToken
     * @return pvg_youtubeapi()
     */
    private function setParseYouTubeParams($type = NULL, $id = NULL, $prevPageToken = NULL, $nextPageToken = NULL) {

        if( !is_null($type) && !is_null($id) ) {

            $this->youtubeId = "{$this->searchType[$type][1]}={$id}";

            if( !is_null($nextPageToken) ) $this->youtubeId .= '&pageToken='.$nextPageToken;
            if( !is_null($prevPageToken) ) $this->youtubeId .= '&pageToken='.$prevPageToken;

            $this->youtubeId .= '&'.$this->searchType[$type][2];
            $this->youtubeType = !is_null($type) ? "{$this->searchType[$type][0]}" : NULL;
        }

        return $this;
    }

    /**
     * Holds multiple youtube ids to be ran at once
     * for single videos per id
     *
     * @param null|array $ids
     * @return pvg_youtubeapi()
     */
    private function setMultipleYTIds($ids = NULL) {
        $this->youtubeMultiId = !empty($ids) && is_array($ids) ? $ids : NULL;
        return $this;
    }

    /**
     * Connect to the youtube service, and retrieve the data.
     *
     * @return array $data
     */
    private function sendRequest() {

        $serviceInfo = NULL;

        $curlOpn = array(
            CURLOPT_RETURNTRANSFER => TRUE, CURLOPT_NOBODY => FALSE,
            CURLOPT_URL => "{$this->apiLink}{$this->youtubeType}?{$this->youtubeId}&orderby={$this->orderBy[3]}&maxResults={$this->maxResults}&key={$this->devKey}",
            CURLOPT_HEADER => FALSE, CURLOPT_ENCODING => "gzip", CURLOPT_SSL_VERIFYPEER => FALSE,
        );

        //Make the curl call to the api service
        $serviceCurl = curl_init();

        curl_setopt_array($serviceCurl, $curlOpn);

        $data = curl_exec($serviceCurl);

        if( !curl_errno($serviceCurl) ) {

            $this->serviceInfo = curl_getinfo($serviceCurl);
        }

        curl_close($serviceCurl);

        $this->results = json_decode($data, TRUE);
        $this->getErrorMessage();
    }

    /**
     * Send Multiple CURL request at once for faster results
     */
    public function sendMultiRequest() {

        $running    = NULL;
        $youtbeData = array();
        $singleCurl = array();
        $multCurl   = curl_multi_init();

        try {

            foreach($this->youtubeMultiId as $index => $videos) {

               $this->setParseYouTubeParams("video", $videos['yt_id']);

               $singleCurl[$index] = curl_init("{$this->apiLink}{$this->youtubeType}?{$this->youtubeId}&order={$this->orderBy[3]}&maxResults={$this->maxResults}&key={$this->devKey}");

               $curlopn[$index] = array(
                   CURLOPT_HEADER         => FALSE,
                   CURLOPT_NOBODY         => FALSE,
                   CURLOPT_RETURNTRANSFER => TRUE,
                   CURLOPT_ENCODING       => 'gzip',
               );

               curl_setopt_array($singleCurl[$index], $curlopn[$index]);
               curl_multi_add_handle($multCurl, $singleCurl[$index]);
            }

            do {

               curl_multi_exec($multCurl, $running);

            }while($running > 0);

            foreach($singleCurl as $key => $value) {

               $data[$key] = curl_multi_getcontent($value);

               curl_multi_remove_handle($multCurl, $value);

               $youtubeResults[$key] = json_decode($data[$key], TRUE);
            }

            curl_multi_close($multCurl);

            foreach($youtubeResults as $key => $items) {

                $this->results['kind'] = $items['kind'];

                if( isset($items['items']) ) {

                    foreach($items['items'] as $data) {

                        $youtbeData['items'][] = $data;
                    }
                }
            }

        }catch(Exception $error) {

           throw new Exception ('Something gone wrong, please try again, or contact the developer', 0, $error);
        }

        //Count all results and get all items in the array
        $this->results['tfound'] = count($youtbeData['items']);
        $this->results['items']  = $youtbeData['items'];

        $this->getErrorMessage();
    }

    /**
     * Catch any errors returned from the youtube service.
     *
     * @return bool|string
     */
    private function getErrorMessage() {

        if (is_array($this->results) && array_key_exists("error", $this->results)) {

            if (isset($this->results['error']['message'])) {

                return "{$this->results['error']['code']}, {$this->results['error']['message']}";
            }
        }

        return FALSE;
    }

    /**
     *
     * @return array
     */
    public function getYoutubeData() {

        if ($this->getResponsetype() === "youtube#video") return $this->getVideo();
        if ($this->getResponsetype() === "youtube#videoListResponse") return $this->getSingleVideos();
        if ($this->getResponsetype() === "youtube#searchListResponse") return $this->searchVideos();
        if ($this->getResponsetype() === "youtube#channelListResponse") return $this->getChannelInfo();
        if ($this->getResponsetype() === "youtube#playlistItemListResponse") return $this->getPlaylist();

        return FALSE;
    }

    /**
     * Get the service information
     *
     * @return array
     */
    public function getServiceInfo() { return $this->serviceInfo; }

    /**
     * Get the full results returned by the service
     *
     * @return array
     */
    public function getResults() { return $this->results; }

    /**
     * @return bool|string
     */
    public function getResponseType() {

        if(isset($this->results['kind'])) return $this->results['kind'];

        return FALSE;
    }

    /**
     * Return the results from the api with out any filters
     *
     * @return array $results
     */
    public function getItems(){

        return array_key_exists("items", $this->results) ? $this->results['items'] : NULL;
    }

    public function getTotalResults() {

        if( isset($this->results['pageInfo']['totalResults']) ) {

            return $this->results['pageInfo']['totalResults'];
        }

        return FALSE;
    }

    /**
     * Return the final results queried from the youtube services.
     *
     * @return array|bool
     */
    public function getPlaylist () {

        $videoData = array();

        if (!is_null($this->getItems())) {

            foreach ($this->getItems() as $index => $items) {

                if( $items['status']['privacyStatus'] == "public") {
                    $videoData[$index]['id']          = $items['contentDetails']['videoId'];
                    $videoData[$index]['title']       = $items['snippet']['title'];
                    $videoData[$index]['published']   = $items['snippet']['publishedAt'];
                    $videoData[$index]['thumbnails']  = $items['snippet']['thumbnails']['high']['url'];
                    $videoData[$index]['description'] = $items['snippet']['description'];
                }
            }
        }

        return $videoData;
    }

    /**
     * This function is used to return a single video in the array
     * similar to getSingleVideos()
     *
     * @return array $videoData
     */
    public function getVideo() {

        $videoData = array();

        if (!is_null($this->getItems())) {

            foreach ($this->getItems() as $index => $items) {

                if ($items['status']['privacyStatus'] === "public") {
                    $videoData[$index]['id']           = $items['id'];
                    $videoData[$index]['channelTitle'] = $items['snippet']['channelTitle'];
                    $videoData[$index]['title']        = $items['snippet']['title'];
                    $videoData[$index]['duration']     = $items['contentDetails']['duration'];
                    $videoData[$index]['likeCount']    = $items['statistics']['likeCount'];
                    $videoData[$index]['viewCount']    = $items['statistics']['viewCount'];
                    $videoData[$index]['published']    = $items['snippet']['publishedAt'];
                    $videoData[$index]['thumbnails']   = $items['snippet']['thumbnails']['high']['url'];
                    $videoData[$index]['description']  = $items['snippet']['description'];
                    $videoData[$index]['dislikeCount'] = $items['statistics']['dislikeCount'];
                }
            }
        }
        return $videoData;
    }
    /**
     * Get single video information
     *
     * @return array $videoData
     */
    public function getSingleVideos() {

        $videoData = array();

        if (isset($this->results['items'])) {

            foreach ($this->results['items'] as $index => $items) {

                if ($items['status']['privacyStatus'] === "public") {
                    $videoData[$index]['id']            = $items['id'];
                    $videoData[$index]['title']         = $items['snippet']['title'];
                    $videoData[$index]['duration']      = $items['contentDetails']['duration'];
                    $videoData[$index]['likeCount']     = $items['statistics']['likeCount'];
                    $videoData[$index]['viewCount']     = $items['statistics']['viewCount'];
                    $videoData[$index]['channelTitle']  = $items['snippet']['channelTitle'];
                    $videoData[$index]['published']     = $items['snippet']['publishedAt'];
                    $videoData[$index]['thumbnails']    = $items['snippet']['thumbnails']['high']['url'];
                    $videoData[$index]['description']   = $items['snippet']['description'];
                    $videoData[$index]['dislikeCount']  = $items['statistics']['dislikeCount'];
                }
            }
        }

        return $videoData;
    }

    /**
     * Search for videos
     */
    public function searchVideos() {

        $videoData = array();

        if (isset($this->results['items'])) {

            foreach($this->results['items'] as $index => $items) {

                if($items['id']['kind'] === "youtube#video" && isset($items['id']['videoId']) ) {
                    $videoData[$index]['id'] = $items['id']['videoId'];
                    $videoData[$index]['title'] = $items['snippet']['title'];
                    $videoData[$index]['published'] = $items['snippet']['publishedAt'];
                    $videoData[$index]['channelId'] = $items['snippet']['channelId'];
                    $videoData[$index]['channelTitle'] = $items['snippet']['channelTitle'];
                    $videoData[$index]['thumbnails'] = $items['snippet']['thumbnails']['high']['url'];
                    $videoData[$index]['description'] = $items['snippet']['description'];
                }
            }
        }

        $videoData['tfound']        = $this->getTotalResults();
        $videoData['prevPageToken'] = isset($this->results['prevPageToken']) ? $this->results['prevPageToken'] : NULL;
        $videoData['nextPageToken'] = isset($this->results['nextPageToken']) ? $this->results['nextPageToken'] : NULL;

        return $videoData;
    }

    /**
     * Extract Channel information (Not yet fully supported)
     * Get channel's uploaded videos for now
     *
     */
    public function getChannelInfo() {

        $videoData = array();

        if (isset($this->results['items'])) {

            //Just get the first one for now.
            $uploadId = $this->results['items'][0]['contentDetails']['relatedPlaylists']['uploads'];

            $data = self::run($uploadId, "playlist");

            if (!is_null($data->getItems())) {

                foreach ($data->getItems() as $index => $items) {

                    if ($items['status']['privacyStatus'] == "public") {
                        $videoData[$index]['id'] = $items['contentDetails']['videoId'];
                        $videoData[$index]['title'] = $items['snippet']['title'];
                        $videoData[$index]['published'] = $items['snippet']['publishedAt'];
                        $videoData[$index]['thumbnails'] = $items['snippet']['thumbnails']['high']['url'];
                        $videoData[$index]['description'] = $items['snippet']['description'];
                    }
                }
            }
        }

        $this->results['pageInfo']['totalResults'] = $data->getTotalResults();

        return $videoData;
    }

    /**
     * Return search options
     *
     * @return array
     */
    public function getSearchOptions() { return $this->searchType; }

    /**
     * Create an object calling the static method from within this realm.
     *
     * @param null|int $id
     * @param null|string $type
     * @param null|string $prevPageToken
     * @param null|string $nextPageToken
     * @param null|string $orderBy
     * @return youtube_v3_api()
     */
    public static function run($id = NULL, $type = NULL, $prevPageToken = NULL, $nextPageToken = NULL, $orderBy = NULL) {

        return new self($id, $type, $prevPageToken, $nextPageToken, $orderBy);
    }
}
